@@ -70,8 +70,8 @@ doctest_path = os.path.abspath('..')
 doctest_test_doctest_blocks = False
 
 doctest_global_setup = """
-from pymongo.connection import Connection
-connection = Connection()
+from pymongo.mongo_client import MongoClient
+connection = MongoClient()
 connection.drop_database("doctest_test")
 db = connection.doctest_test
 """
@@ -81,6 +81,7 @@ db = connection.doctest_test
 # The theme to use for HTML and HTML Help pages.  Major themes that come with
 # Sphinx are currently 'default' and 'sphinxdoc'.
 html_theme = 'default'
+html_theme_options = {'collapsiblesidebar': True}
 
 # The name for this set of Sphinx documents.  If None, it defaults to
 # "<project> v<release> documentation".
