@@ -62,6 +62,14 @@ def validate_positive_integer(option, value):
     return val
 
 
+def validate_positive_integer_or_none(option, value):
+    """Validate that 'value' is a positive integer or None.
+    """
+    if value is None:
+        return value
+    return validate_positive_integer(option, value)
+
+
 def validate_basestring(option, value):
     """Validates that 'value' is an instance of `basestring`.
     """
@@ -114,9 +122,14 @@ def validate_timeout_or_none(option, value):
 def validate_read_preference(dummy, value):
     """Validate read preference for a ReplicaSetConnection.
     """
-    if value not in read_preferences.modes:
+    if value in read_preferences.modes:
+        return value
+
+    # Also allow string form of enum for uri_parser
+    try:
+        return read_preferences.mongos_enum(value)
+    except ValueError:
         raise ConfigurationError("Not a valid read preference")
-    return value
 
 
 def validate_tag_sets(dummy, value):
@@ -157,6 +170,7 @@ VALIDATORS = {
     'connecttimeoutms': validate_timeout_or_none,
     'sockettimeoutms': validate_timeout_or_none,
     'ssl': validate_boolean,
+    'readpreference': validate_read_preference,
     'read_preference': validate_read_preference,
     'tag_sets': validate_tag_sets,
     'secondaryacceptablelatencyms': validate_positive_float,
@@ -253,7 +267,7 @@ class BaseObject(object):
         for option, value in options.iteritems():
             if option in ('slave_okay', 'slaveok'):
                 self.__slave_okay = validate_boolean(option, value)
-            elif option == 'read_preference':
+            elif option in ('read_preference', "readpreference"):
                 self.__read_pref = validate_read_preference(option, value)
             elif option == 'tag_sets':
                 self.__tag_sets = validate_tag_sets(option, value)
