@@ -416,7 +416,7 @@ class Cursor(object):
 
         An :class:`IndexError` will be raised if the index is negative
         or greater than the amount of documents in this cursor. Any
-        limit applied to this cursor will be ignored.
+        limit previously applied to this cursor will be ignored.
 
         To get a slice of documents use a slice index, e.g.::
 
@@ -854,7 +854,7 @@ class Cursor(object):
             return memo.get(val_id)
         memo[val_id] = y
         for key, value in x.iteritems():
-            if isinstance(value, dict):
+            if isinstance(value, dict) and not isinstance(value, SON):
                 value = self.__deepcopy(value, memo)
             elif not isinstance(value, RE_TYPE):
                 value = copy.deepcopy(value, memo)
