@@ -208,6 +208,10 @@ class MongoClient(common.BaseObject):
         self.__is_primary = False
         self.__is_mongos = False
 
+        # _pool_class option is for deep customization of PyMongo, e.g. Motor.
+        # SHOULD NOT BE USED BY DEVELOPERS EXTERNAL TO 10GEN.
+        pool_class = kwargs.pop('_pool_class', pool.Pool)
+
         options = {}
         for option, value in kwargs.iteritems():
             option, value = common.validate(option, value)
@@ -235,23 +239,14 @@ class MongoClient(common.BaseObject):
                                      "2.6 you must install the ssl package "
                                      "from PyPI.")
 
-        if options.get('use_greenlets', False):
-            if not pool.have_greenlet:
-                raise ConfigurationError(
-                    "The greenlet module is not available. "
-                    "Install the greenlet package from PyPI."
-                )
-            self.pool_class = pool.GreenletPool
-        else:
-            self.pool_class = pool.Pool
-
-        self.__pool = self.pool_class(
+        self.__use_greenlets = options.get('use_greenlets', False)
+        self.__pool = pool_class(
             None,
             self.__max_pool_size,
             self.__net_timeout,
             self.__conn_timeout,
-            self.__use_ssl
-        )
+            self.__use_ssl,
+            use_greenlets=self.__use_greenlets)
 
         self.__document_class = document_class
         self.__tz_aware = common.validate_boolean('tz_aware', tz_aware)
@@ -356,9 +351,6 @@ class MongoClient(common.BaseObject):
 
     def __check_auth(self, sock_info):
         """Authenticate using cached database credentials.
-
-        If credentials for the 'admin' database are available only
-        this database is authenticated, since this gives global access.
         """
         authset = sock_info.authset
         names = set(self.__auth_credentials.iterkeys())
@@ -373,19 +365,10 @@ class MongoClient(common.BaseObject):
                 pass
             authset.discard(dbname)
 
-        # Once logged into the admin database we can access anything.
-        if "admin" in authset:
-            return
-
-        if "admin" in self.__auth_credentials:
-            username, password = self.__auth_credentials["admin"]
-            self.__auth(sock_info, 'admin', username, password)
-            authset.add('admin')
-        else:
-            for db_name in names - authset:
-                user, pwd = self.__auth_credentials[db_name]
-                self.__auth(sock_info, db_name, user, pwd)
-                authset.add(db_name)
+        for db_name in names - authset:
+            user, pwd = self.__auth_credentials[db_name]
+            self.__auth(sock_info, db_name, user, pwd)
+            authset.add(db_name)
 
     @property
     def host(self):
@@ -436,6 +419,15 @@ class MongoClient(common.BaseObject):
         """
         return self.__max_pool_size
 
+    @property
+    def use_greenlets(self):
+        """Whether calling :meth:`start_request` assigns greenlet-local,
+        rather than thread-local, sockets.
+
+        .. versionadded:: 2.4.2
+        """
+        return self.__use_greenlets
+
     @property
     def nodes(self):
         """List of all known nodes.
