@@ -47,6 +47,7 @@ from pymongo import (common,
                      helpers,
                      message,
                      pool,
+                     thread_util,
                      uri_parser)
 from pymongo.read_preferences import (
     ReadPreference, select_member, modes, MovingAverage)
@@ -61,6 +62,11 @@ EMPTY = b("")
 MAX_BSON_SIZE = 4 * 1024 * 1024
 MAX_RETRY = 3
 
+# Member states
+PRIMARY = 1
+SECONDARY = 2
+OTHER = 3
+
 MONITORS = set()
 
 def register_monitor(monitor):
@@ -194,13 +200,27 @@ class Member(object):
         self.update(ismaster_response, ping_time)
 
     def update(self, ismaster_response, ping_time):
-        self.is_primary = ismaster_response['ismaster']
+        if ismaster_response['ismaster']:
+            self.state = PRIMARY
+        elif ismaster_response.get('secondary'):
+            self.state = SECONDARY
+        else:
+            self.state = OTHER
+
         self.max_bson_size = ismaster_response.get(
             'maxBsonObjectSize', MAX_BSON_SIZE)
         self.tags = ismaster_response.get('tags', {})
         self.record_ping_time(ping_time)
         self.up = True
 
+    @property
+    def is_primary(self):
+        return self.state == PRIMARY
+
+    @property
+    def is_secondary(self):
+        return self.state == SECONDARY
+
     def get_avg_ping_time(self):
         """Get a moving average of this member's ping times
         """
@@ -217,10 +237,12 @@ class Member(object):
         if mode == ReadPreference.PRIMARY and not self.is_primary:
             return False
 
-        if mode == ReadPreference.SECONDARY and self.is_primary:
+        if mode == ReadPreference.SECONDARY and not self.is_secondary:
             return False
 
-        return True
+        # If we're not primary or secondary, then we're in a state like
+        # RECOVERING and we don't match any mode
+        return self.is_primary or self.is_secondary
 
     def matches_tags(self, tags):
         """Return True if this member's tags are a superset of the passed-in
@@ -338,8 +360,8 @@ class MongoReplicaSetClient(common.BaseObject):
             there are no consistency guarantees. Default to ``False``.
           - `use_greenlets`: If ``True``, use a background Greenlet instead of
             a background thread to monitor state of replica set. Additionally,
-            :meth:`start_request()` will ensure that the current greenlet uses
-            the same socket for all operations until :meth:`end_request()`.
+            :meth:`start_request()` assigns a greenlet-local, rather than
+            thread-local, socket.
             `use_greenlets` with :class:`MongoReplicaSetClient` requires
             `Gevent <http://gevent.org/>`_ to be installed.
           - `host`: For compatibility with :class:`~mongo_client.MongoClient`.
@@ -355,7 +377,6 @@ class MongoReplicaSetClient(common.BaseObject):
         self.__hosts = None
         self.__arbiters = set()
         self.__writer = None
-        self.__readers = []
         self.__members = {}
         self.__index_cache = {}
         self.__auth_credentials = {}
@@ -388,23 +409,28 @@ class MongoReplicaSetClient(common.BaseObject):
         else:
             self.__seeds.update(uri_parser.split_hosts(host, port))
 
+        # _pool_class and _monitor_class are for deep customization of PyMongo,
+        # e.g. Motor. SHOULD NOT BE USED BY DEVELOPERS EXTERNAL TO 10GEN.
+        self.pool_class = kwargs.pop('_pool_class', pool.Pool)
+        monitor_class = kwargs.pop('_monitor_class', None)
+
         for option, value in kwargs.iteritems():
             option, value = common.validate(option, value)
             self.__opts[option] = value
         self.__opts.update(options)
 
-        if self.__opts.get('use_greenlets', False):
-            if not have_gevent:
-                raise ConfigurationError(
-                    "The gevent module is not available. "
-                    "Install the gevent package from PyPI."
-                )
-            self.pool_class = pool.GreenletPool
-        else:
-            self.pool_class = pool.Pool
+        self.__use_greenlets = self.__opts.get('use_greenlets', False)
+        if self.__use_greenlets and not have_gevent:
+            raise ConfigurationError(
+                "The gevent module is not available. "
+                "Install the gevent package from PyPI.")
+
+        self.__request_counter = thread_util.Counter(self.__use_greenlets)
 
         self.__auto_start_request = self.__opts.get('auto_start_request', False)
-        self.__in_request = self.__auto_start_request
+        if self.__auto_start_request:
+            self.start_request()
+
         self.__reset_pinned_hosts()
         self.__name = self.__opts.get('replicaset')
         if not self.__name:
@@ -442,7 +468,9 @@ class MongoReplicaSetClient(common.BaseObject):
                 raise ConfigurationError("authentication failed")
 
         # Start the monitor after we know the configuration is correct.
-        if self.__opts.get('use_greenlets', False):
+        if monitor_class:
+            self.__monitor = monitor_class(self)
+        elif self.__use_greenlets:
             self.__monitor = MonitorGreenlet(self)
         else:
             self.__monitor = MonitorThread(self)
@@ -524,9 +552,6 @@ class MongoReplicaSetClient(common.BaseObject):
 
     def __check_auth(self, sock_info):
         """Authenticate using cached database credentials.
-
-        If credentials for the 'admin' database are available only
-        this database is authenticated, since this gives global access.
         """
         authset = sock_info.authset
         names = set(self.__auth_credentials.iterkeys())
@@ -541,19 +566,10 @@ class MongoReplicaSetClient(common.BaseObject):
                 pass
             authset.discard(dbname)
 
-        # Once logged into the admin database we can access anything.
-        if "admin" in authset:
-            return
-
-        if "admin" in self.__auth_credentials:
-            username, password = self.__auth_credentials["admin"]
-            self.__auth(sock_info, 'admin', username, password)
-            authset.add('admin')
-        else:
-            for db_name in names - authset:
-                user, pwd = self.__auth_credentials[db_name]
-                self.__auth(sock_info, db_name, user, pwd)
-                authset.add(db_name)
+        for db_name in names - authset:
+            user, pwd = self.__auth_credentials[db_name]
+            self.__auth(sock_info, db_name, user, pwd)
+            authset.add(db_name)
 
     @property
     def seeds(self):
@@ -581,7 +597,9 @@ class MongoReplicaSetClient(common.BaseObject):
     def secondaries(self):
         """The secondary members known to this connection.
         """
-        return set(self.__readers)
+        return set([
+            host for host, member in self.__members.items()
+            if member.is_secondary])
 
     @property
     def arbiters(self):
@@ -610,6 +628,15 @@ class MongoReplicaSetClient(common.BaseObject):
         """
         return self.__max_pool_size
 
+    @property
+    def use_greenlets(self):
+        """Whether calling :meth:`start_request` assigns greenlet-local,
+        rather than thread-local, sockets.
+
+        .. versionadded:: 2.4.2
+        """
+        return self.__use_greenlets
+
     def get_document_class(self):
         """document_class getter"""
         return self.__document_class
@@ -683,7 +710,10 @@ class MongoReplicaSetClient(common.BaseObject):
         """
         connection_pool = self.pool_class(
             host, self.__max_pool_size, self.__net_timeout, self.__conn_timeout,
-            self.__use_ssl)
+            self.__use_ssl, use_greenlets=self.__use_greenlets)
+
+        if self.in_request():
+            connection_pool.start_request()
 
         sock_info = connection_pool.get_socket()
         try:
@@ -701,7 +731,6 @@ class MongoReplicaSetClient(common.BaseObject):
         """Update the mapping of (host, port) pairs to connection pools.
         """
         primary = None
-        secondaries = []
         for host in self.__hosts:
             member, sock_info = None, None
             try:
@@ -724,18 +753,14 @@ class MongoReplicaSetClient(common.BaseObject):
                     member.pool.discard_socket(sock_info)
                     self.__members.pop(member.host, None)
                 continue
-            # Only use hosts that are currently in 'secondary' state
-            # as readers.
-            if res['secondary']:
-                secondaries.append(host)
-            elif res['ismaster']:
+
+            if res['ismaster']:
                 primary = host
 
         if primary != self.__writer:
             self.__reset_pinned_hosts()
 
         self.__writer = primary
-        self.__readers = secondaries
 
     def __schedule_refresh(self):
         self.__monitor.schedule_refresh()
@@ -754,7 +779,7 @@ class MongoReplicaSetClient(common.BaseObject):
         self.__threadlocal.host = None
 
     def __reset_pinned_hosts(self):
-        if self.__opts.get('use_greenlets', False):
+        if self.__use_greenlets:
             self.__threadlocal = gevent_local()
         else:
             self.__threadlocal = threading.local()
@@ -875,8 +900,7 @@ class MongoReplicaSetClient(common.BaseObject):
     def __socket(self, member):
         """Get a SocketInfo from the pool.
         """
-        if self.__auto_start_request:
-            # No effect if a request already started
+        if self.auto_start_request and not self.in_request():
             self.start_request()
 
         sock_info = member.pool.get_socket()
@@ -886,12 +910,14 @@ class MongoReplicaSetClient(common.BaseObject):
         return sock_info
 
     def disconnect(self):
-        """Disconnect from the replica set primary.
+        """Disconnect from the replica set primary and refresh our view of
+        the replica set.
         """
         member = self.__members.get(self.__writer)
         if member:
             member.pool.reset()
         self.__writer = None
+        self.__schedule_refresh()
 
     def close(self):
         """Close this connection instance.
@@ -1235,10 +1261,16 @@ class MongoReplicaSetClient(common.BaseObject):
            The :class:`~pymongo.pool.Request` return value.
            :meth:`start_request` previously returned None
         """
-        for member in self.__members.values():
-            member.pool.start_request()
+        # We increment our request counter's thread- or greenlet-local value
+        # for every call to start_request; however, we only call each pool's
+        # start_request once to start a request, and call each pool's
+        # end_request once to end it. We don't let pools' request counters
+        # exceed 1. This keeps things sane when we create and delete pools
+        # within a request.
+        if 1 == self.__request_counter.inc():
+            for member in self.__members.values():
+                member.pool.start_request()
 
-        self.__in_request = True
         return pool.Request(self)
 
     def in_request(self):
@@ -1246,7 +1278,7 @@ class MongoReplicaSetClient(common.BaseObject):
         :meth:`end_request`, or if `auto_start_request` is True and
         :meth:`end_request` has not been called in this thread or greenlet.
         """
-        return self.__in_request
+        return bool(self.__request_counter.get())
 
     def end_request(self):
         """Undo :meth:`start_request` and allow this thread's connections to
@@ -1262,11 +1294,12 @@ class MongoReplicaSetClient(common.BaseObject):
         in the middle of a sequence of operations in which ordering is
         important. This could lead to unexpected results.
         """
-        for member in self.__members.values():
-            member.pool.end_request()
+        if 0 == self.__request_counter.dec():
+            for member in self.__members.values():
+                # No effect if not in a request
+                member.pool.end_request()
 
-        self.__in_request = False
-        self.__unpin_host()
+            self.__unpin_host()
 
     def __eq__(self, other):
         # XXX: Implement this?
@@ -1389,10 +1422,9 @@ class MongoReplicaSetClient(common.BaseObject):
         if from_host is not None:
             command["fromhost"] = from_host
 
-        in_request = self.in_request()
         try:
-            if not in_request:
-                self.start_request()
+            self.start_request()
+
             if username is not None:
                 nonce = self.admin.command("copydbgetnonce",
                                            fromhost=from_host)["nonce"]
@@ -1402,5 +1434,4 @@ class MongoReplicaSetClient(common.BaseObject):
 
             return self.admin.command("copydb", **command)
         finally:
-            if not in_request:
-                self.end_request()
+            self.end_request()
