@@ -28,19 +28,27 @@ from ha_tools import use_greenlets
 
 from pymongo import (ReplicaSetConnection,
                      ReadPreference)
-from pymongo.read_preferences import modes
 from pymongo.mongo_replica_set_client import Member, Monitor
 from pymongo.mongo_client import _partition_node
 from pymongo.connection import Connection
-from pymongo.errors import AutoReconnect, OperationFailure
+from pymongo.errors import AutoReconnect, OperationFailure, ConnectionFailure
 
 from test import utils
+from test.utils import one
 
 
 # Override default 30-second interval for faster testing
 Monitor._refresh_interval = MONITOR_INTERVAL = 0.5
 
 
+# To make the code terser, copy modes into module scope
+PRIMARY = ReadPreference.PRIMARY
+PRIMARY_PREFERRED = ReadPreference.PRIMARY_PREFERRED
+SECONDARY = ReadPreference.SECONDARY
+SECONDARY_PREFERRED = ReadPreference.SECONDARY_PREFERRED
+NEAREST = ReadPreference.NEAREST
+
+
 class TestDirectConnection(unittest.TestCase):
 
     def setUp(self):
@@ -70,11 +78,11 @@ class TestDirectConnection(unittest.TestCase):
 
         # A Connection succeeds no matter the read preference
         for kwargs in [
-            {'read_preference': ReadPreference.PRIMARY},
-            {'read_preference': ReadPreference.PRIMARY_PREFERRED},
-            {'read_preference': ReadPreference.SECONDARY},
-            {'read_preference': ReadPreference.SECONDARY_PREFERRED},
-            {'read_preference': ReadPreference.NEAREST},
+            {'read_preference': PRIMARY},
+            {'read_preference': PRIMARY_PREFERRED},
+            {'read_preference': SECONDARY},
+            {'read_preference': SECONDARY_PREFERRED},
+            {'read_preference': NEAREST},
             {'slave_okay': True}
         ]:
             conn = Connection(primary_host,
@@ -98,7 +106,7 @@ class TestDirectConnection(unittest.TestCase):
 
             # Direct connection to secondary can be queried with any read pref
             # but PRIMARY
-            if kwargs.get('read_preference') != ReadPreference.PRIMARY:
+            if kwargs.get('read_preference') != PRIMARY:
                 self.assertTrue(conn.pymongo_test.test.find_one())
             else:
                 self.assertRaises(
@@ -110,7 +118,7 @@ class TestDirectConnection(unittest.TestCase):
             try:
                 conn.pymongo_test.test.insert({}, safe=False)
             except AutoReconnect, e:
-                self.assertEqual('not master', e.message)
+                self.assertEqual('not master', e.args[0])
             else:
                 self.fail(
                     'Unacknowledged insert into secondary connection %s should'
@@ -126,7 +134,7 @@ class TestDirectConnection(unittest.TestCase):
             try:
                 conn.pymongo_test.test.insert({}, safe=False)
             except AutoReconnect, e:
-                self.assertEqual('not master', e.message)
+                self.assertEqual('not master', e.args[0])
             else:
                 self.fail(
                     'Unacknowledged insert into arbiter connection %s should'
@@ -152,41 +160,134 @@ class TestPassiveAndHidden(unittest.TestCase):
     def test_passive_and_hidden(self):
         self.c = ReplicaSetConnection(
             self.seed, replicaSet=self.name, use_greenlets=use_greenlets)
-        db = self.c.pymongo_test
-        w = len(self.c.secondaries) + 1
-        db.test.remove({}, safe=True, w=w)
-        db.test.insert({'foo': 'bar'}, safe=True, w=w)
 
         passives = ha_tools.get_passives()
         passives = [_partition_node(member) for member in passives]
-        hidden = ha_tools.get_hidden_members()
-        hidden = [_partition_node(member) for member in hidden]
         self.assertEqual(self.c.secondaries, set(passives))
 
-        for mode in (
-            ReadPreference.SECONDARY, ReadPreference.SECONDARY_PREFERRED
-        ):
-            db.read_preference = mode
-            for _ in xrange(10):
-                cursor = db.test.find()
-                cursor.next()
-                self.assertTrue(cursor._Cursor__connection_id in passives)
-                self.assertTrue(cursor._Cursor__connection_id not in hidden)
+        for mode in SECONDARY, SECONDARY_PREFERRED:
+            utils.assertReadFromAll(self, self.c, passives, mode)
 
         ha_tools.kill_members(ha_tools.get_passives(), 2)
         sleep(2 * MONITOR_INTERVAL)
-        db.read_preference = ReadPreference.SECONDARY_PREFERRED
-
-        for _ in xrange(10):
-            cursor = db.test.find()
-            cursor.next()
-            self.assertEqual(cursor._Cursor__connection_id, self.c.primary)
+        utils.assertReadFrom(self, self.c, self.c.primary, SECONDARY_PREFERRED)
 
     def tearDown(self):
         self.c.close()
         ha_tools.kill_all_members()
 
 
+class TestMonitorRemovesRecoveringMember(unittest.TestCase):
+    # Members in STARTUP2 or RECOVERING states are shown in the primary's
+    # isMaster response, but aren't secondaries and shouldn't be read from.
+    # Verify that if a secondary goes into RECOVERING mode, the Monitor removes
+    # it from the set of readers.
+
+    def setUp(self):
+        members = [{}, {'priority': 0}, {'priority': 0}]
+        res = ha_tools.start_replica_set(members)
+        self.seed, self.name = res
+
+    def test_monitor_removes_recovering_member(self):
+        self.c = ReplicaSetConnection(
+            self.seed, replicaSet=self.name, use_greenlets=use_greenlets,
+            auto_start_request=False)
+
+        secondaries = ha_tools.get_secondaries()
+
+        for mode in SECONDARY, SECONDARY_PREFERRED:
+            partitioned_secondaries = [_partition_node(s) for s in secondaries]
+            utils.assertReadFromAll(self, self.c, partitioned_secondaries, mode)
+
+        secondary, recovering_secondary = secondaries
+        ha_tools.set_maintenance(recovering_secondary, True)
+        sleep(2 * MONITOR_INTERVAL)
+
+        for mode in SECONDARY, SECONDARY_PREFERRED:
+            # Don't read from recovering member
+            utils.assertReadFrom(self, self.c, _partition_node(secondary), mode)
+
+    def tearDown(self):
+        self.c.close()
+        ha_tools.kill_all_members()
+
+
+class TestTriggeredRefresh(unittest.TestCase):
+    # Verify that if a secondary goes into RECOVERING mode or if the primary
+    # changes, the next exception triggers an immediate refresh.
+
+    def setUp(self):
+        members = [{}, {}]
+        res = ha_tools.start_replica_set(members)
+        self.seed, self.name = res
+
+        # Disable periodic refresh
+        Monitor._refresh_interval = 1e6
+
+    def test_recovering_member_triggers_refresh(self):
+        # To test that find_one() and count() trigger immediate refreshes,
+        # we'll create a separate connection for each
+        self.c_find_one, self.c_count = [
+            ReplicaSetConnection(
+                self.seed, replicaSet=self.name, use_greenlets=use_greenlets,
+                auto_start_request=False, read_preference=SECONDARY)
+            for _ in xrange(2)]
+
+        # We've started the primary and one secondary
+        primary = ha_tools.get_primary()
+        secondary = ha_tools.get_secondaries()[0]
+
+        # Pre-condition: just make sure they all connected OK
+        for c in self.c_find_one, self.c_count:
+            self.assertEqual(one(c.secondaries), _partition_node(secondary))
+
+        ha_tools.set_maintenance(secondary, True)
+
+        # Trigger a refresh in various ways
+        self.assertRaises(AutoReconnect, self.c_find_one.test.test.find_one)
+        self.assertRaises(AutoReconnect, self.c_count.test.test.count)
+
+        # Wait for the immediate refresh to complete - we're not waiting for
+        # the periodic refresh, which has been disabled
+        sleep(1)
+
+        for c in self.c_find_one, self.c_count:
+            self.assertFalse(c.secondaries)
+            self.assertEqual(_partition_node(primary), c.primary)
+
+    def test_stepdown_triggers_refresh(self):
+        c_find_one = ReplicaSetConnection(
+            self.seed, replicaSet=self.name, use_greenlets=use_greenlets,
+            auto_start_request=False)
+
+        # We've started the primary and one secondary
+        primary = ha_tools.get_primary()
+        secondary = ha_tools.get_secondaries()[0]
+        self.assertEqual(
+            one(c_find_one.secondaries), _partition_node(secondary))
+
+        ha_tools.stepdown_primary()
+
+        # Make sure the stepdown completes
+        sleep(1)
+
+        # Trigger a refresh
+        self.assertRaises(AutoReconnect, c_find_one.test.test.find_one)
+
+        # Wait for the immediate refresh to complete - we're not waiting for
+        # the periodic refresh, which has been disabled
+        sleep(1)
+
+        # We've detected the stepdown
+        self.assertTrue(
+            not c_find_one.primary
+            or primary != _partition_node(c_find_one.primary))
+
+    def tearDown(self):
+        Monitor._refresh_interval = MONITOR_INTERVAL
+        ha_tools.kill_all_members()
+
+
 class TestHealthMonitor(unittest.TestCase):
 
     def setUp(self):
@@ -324,7 +425,7 @@ class TestReadWithFailover(unittest.TestCase):
         db.test.insert([{'foo': i} for i in xrange(10)], safe=True, w=w)
         self.assertEqual(10, db.test.count())
 
-        db.read_preference = ReadPreference.SECONDARY_PREFERRED
+        db.read_preference = SECONDARY_PREFERRED
         cursor = db.test.find().batch_size(5)
         cursor.next()
         self.assertEqual(5, cursor._Cursor__retrieved)
@@ -420,13 +521,7 @@ class TestReadPreference(unittest.TestCase):
             host, port = node
             return '%s:%s' % (host, port)
 
-        # To make the code terser, copy modes and hosts into local scope
-        PRIMARY = ReadPreference.PRIMARY
-        PRIMARY_PREFERRED = ReadPreference.PRIMARY_PREFERRED
-        SECONDARY = ReadPreference.SECONDARY
-        SECONDARY_PREFERRED = ReadPreference.SECONDARY_PREFERRED
-        NEAREST = ReadPreference.NEAREST
-
+        # To make the code terser, copy hosts into local scope
         primary = self.primary
         secondary = self.secondary
         other_secondary = self.other_secondary
@@ -673,7 +768,7 @@ class TestReplicaSetAuth(unittest.TestCase):
         # Make sure we can still authenticate
         self.assertTrue(self.db.authenticate('user', 'userpass'))
         # And still query.
-        self.db.read_preference = ReadPreference.PRIMARY_PREFERRED
+        self.db.read_preference = PRIMARY_PREFERRED
         self.assertEqual('bar', self.db.foo.find_one()['foo'])
 
     def tearDown(self):
@@ -762,6 +857,62 @@ class TestMongosHighAvailability(unittest.TestCase):
         ha_tools.kill_all_members()
 
 
+class TestReplicaSetRequest(unittest.TestCase):
+    def setUp(self):
+        members = [{}, {}, {'arbiterOnly': True}]
+        res = ha_tools.start_replica_set(members)
+        self.c = ReplicaSetConnection(res[0], replicaSet=res[1],
+                                      use_greenlets=use_greenlets)
+
+    def test_request_during_failover(self):
+        primary = _partition_node(ha_tools.get_primary())
+        secondary = _partition_node(ha_tools.get_random_secondary())
+
+        self.assertTrue(self.c.auto_start_request)
+        self.assertTrue(self.c.in_request())
+
+        primary_pool = self.c._MongoReplicaSetClient__members[primary].pool
+        secondary_pool = self.c._MongoReplicaSetClient__members[secondary].pool
+
+        # Trigger start_request on primary pool
+        utils.assertReadFrom(self, self.c, primary, PRIMARY)
+        self.assertTrue(primary_pool.in_request())
+
+        # Fail over
+        ha_tools.kill_primary()
+        patience_seconds = 60
+        for _ in range(patience_seconds):
+            sleep(1)
+            try:
+                if ha_tools.ha_tools_debug:
+                    print 'Waiting for failover'
+                if ha_tools.get_primary():
+                    # We have a new primary
+                    break
+            except ConnectionFailure:
+                pass
+        else:
+            self.fail("Problem with test: No new primary after %s seconds"
+                % patience_seconds)
+
+        try:
+            # Trigger start_request on secondary_pool, which is becoming new
+            # primary
+            self.c.test.test.find_one()
+        except AutoReconnect:
+            # We've noticed the failover now
+            pass
+
+        # The old secondary is now primary
+        utils.assertReadFrom(self, self.c, secondary, PRIMARY)
+        self.assertTrue(self.c.in_request())
+        self.assertTrue(secondary_pool.in_request())
+
+    def tearDown(self):
+        self.c.close()
+        ha_tools.kill_all_members()
+
+
 if __name__ == '__main__':
     if use_greenlets:
         print('Using Gevent')
