@@ -66,10 +66,6 @@ class TestCollection(unittest.TestCase):
         self.db = None
         self.connection = None
 
-        # Try to diagnose intermittent failure in tests on Jenkins
-        cx = get_connection()
-        self.assertEqual(0, cx.test.test_unique_threaded.count())
-
     def test_collection(self):
         self.assertRaises(TypeError, Collection, self.db, 5)
 
@@ -386,6 +382,23 @@ class TestCollection(unittest.TestCase):
             "type": "restaurant"
         }, results[0])
 
+    def test_index_text(self):
+        if not version.at_least(self.connection, (2, 3, 2)):
+            raise SkipTest("Text search requires server >=2.3.2.")
+
+        self.connection.admin.command('setParameter', '*',
+                                      textSearchEnabled=True)
+
+        db = self.db
+        db.test.drop_indexes()
+        self.assertEqual("t_text", db.test.create_index([("t", "text")]))
+        index_info = db.test.index_information()["t_text"]
+        self.assertTrue("weights" in index_info)
+        db.test.drop_indexes()
+
+        self.connection.admin.command('setParameter', '*',
+                                      textSearchEnabled=False)
+
     def test_index_sparse(self):
         db = self.db
         db.test.drop_indexes()
@@ -1592,7 +1605,7 @@ class TestCollection(unittest.TestCase):
                 self.assertEqual(3, result.find_one({"_id": "cat"})["value"])
                 self.assertEqual(2, result.find_one({"_id": "dog"})["value"])
                 self.assertEqual(1, result.find_one({"_id": "mouse"})["value"])
-                self.connection.mrtestdb.mrunittests.drop()
+                self.connection.drop_database('mrtestdb')
 
         full_result = db.test.map_reduce(map, reduce,
                                          out='mrunittests', full_response=True)
