@@ -33,6 +33,7 @@ from bson.tz_util import utc
 from pymongo.connection import Connection
 from pymongo.database import Database
 from pymongo.pool import SocketInfo
+from pymongo import thread_util
 from pymongo.errors import (ConfigurationError,
                             ConnectionFailure,
                             InvalidName,
@@ -113,6 +114,13 @@ class TestConnection(unittest.TestCase, TestRequestMixin):
         self.assertEqual(set([(self.host, self.port)]),
                          Connection(self.host, self.port).nodes)
 
+    def test_use_greenlets(self):
+        self.assertFalse(Connection(self.host, self.port).use_greenlets)
+        if thread_util.have_greenlet:
+            self.assertTrue(
+                Connection(
+                    self.host, self.port, use_greenlets=True).use_greenlets)
+
     def test_get_db(self):
         connection = Connection(self.host, self.port)
 
@@ -235,7 +243,7 @@ class TestConnection(unittest.TestCase, TestRequestMixin):
 
     def test_disconnect(self):
         c = Connection(self.host, self.port)
-        coll = c.foo.bar
+        coll = c.pymongo_test.bar
 
         c.disconnect()
         c.disconnect()
@@ -537,8 +545,8 @@ self.assertEqual(0, len(conn._MongoClient__pool.sockets))
         exec """
 with get_connection() as connection:
     self.assertEqual("bar", connection.pymongo_test.test.find_one()["foo"])
-    # Calling conn.close() has reset the pool
-    self.assertEqual(0, len(connection._MongoClient__pool.sockets))
+# Calling conn.close() has reset the pool
+self.assertEqual(0, len(connection._MongoClient__pool.sockets))
 """
 
     def test_with_start_request(self):
@@ -604,7 +612,7 @@ with conn.start_request() as request:
         self.assertDifferentSock(pool)
 
         # Trigger auto_start_request
-        conn.db.test.find_one()
+        conn.pymongo_test.test.find_one()
         self.assertRequestSocket(pool)
         self.assertSameSock(pool)
 
