@@ -25,6 +25,7 @@ from pymongo.uri_parser import (_partition,
                                 split_options,
                                 parse_uri)
 from pymongo.errors import ConfigurationError, InvalidURI
+from pymongo import ReadPreference
 
 
 class TestURI(unittest.TestCase):
@@ -268,5 +269,10 @@ class TestURI(unittest.TestCase):
                          parse_uri("mongodb://fred:foobar@localhost/"
                                    "test.yield_historical.in?slaveok=true"))
 
+        res = copy.deepcopy(orig)
+        res['options'] = {'readpreference': ReadPreference.SECONDARY}
+        self.assertEqual(res,
+                         parse_uri("mongodb://localhost/?readPreference=secondary"))
+
 if __name__ == "__main__":
     unittest.main()
