@@ -15,10 +15,16 @@
 """Utilities for testing pymongo
 """
 
+import threading
+
 from pymongo.errors import AutoReconnect
 from pymongo.pool import NO_REQUEST, NO_SOCKET_YET, SocketInfo
 
 
+def one(s):
+    """Get one element of a set"""
+    return iter(s).next()
+
 def delay(sec):
     # Javascript sleep() only available in MongoDB since version ~1.9
     return '''function() {
@@ -67,6 +73,116 @@ def assertRaisesExactly(cls, fn, *args, **kwargs):
     else:
         raise AssertionError("%s not raised" % cls)
 
+def looplet(greenlets):
+    """World's smallest event loop; run until all greenlets are done
+    """
+    while True:
+        done = True
+
+        for g in greenlets:
+            if not g.dead:
+                done = False
+                g.switch()
+
+        if done:
+            return
+
+class RendezvousThread(threading.Thread):
+    """A thread that starts and pauses at a rendezvous point before resuming.
+    To be used in tests that must ensure that N threads are all alive
+    simultaneously, regardless of thread-scheduling's vagaries.
+
+    1. Write a subclass of RendezvousThread and override before_rendezvous
+      and / or after_rendezvous.
+    2. Create a state with RendezvousThread.shared_state(N)
+    3. Start N of your subclassed RendezvousThreads, passing the state to each
+      one's __init__
+    4. In the main thread, call RendezvousThread.wait_for_rendezvous
+    5. Test whatever you need to test while threads are paused at rendezvous
+      point
+    6. In main thread, call RendezvousThread.resume_after_rendezvous
+    7. Join all threads from main thread
+    8. Assert that all threads' "passed" attribute is True
+    9. Test post-conditions
+    """
+    class RendezvousState(object):
+        def __init__(self, nthreads):
+            # Number of threads total
+            self.nthreads = nthreads
+    
+            # Number of threads that have arrived at rendezvous point
+            self.arrived_threads = 0
+            self.arrived_threads_lock = threading.Lock()
+    
+            # Set when all threads reach rendezvous
+            self.ev_arrived = threading.Event()
+    
+            # Set by resume_after_rendezvous() so threads can continue.
+            self.ev_resume = threading.Event()
+            
+
+    @classmethod
+    def create_shared_state(cls, nthreads):
+        return RendezvousThread.RendezvousState(nthreads)
+
+    def before_rendezvous(self):
+        """Overridable: Do this before the rendezvous"""
+        pass
+
+    def after_rendezvous(self):
+        """Overridable: Do this after the rendezvous. If it throws no exception,
+        `passed` is set to True
+        """
+        pass
+
+    @classmethod
+    def wait_for_rendezvous(cls, state):
+        """Wait for all threads to reach rendezvous and pause there"""
+        state.ev_arrived.wait(10)
+        assert state.ev_arrived.isSet(), "Thread timeout"
+        assert state.nthreads == state.arrived_threads
+
+    @classmethod
+    def resume_after_rendezvous(cls, state):
+        """Tell all the paused threads to continue"""
+        state.ev_resume.set()
+
+    def __init__(self, state):
+        """Params:
+          `state`: A shared state object from RendezvousThread.shared_state()
+        """
+        super(RendezvousThread, self).__init__()
+        self.state = state
+        self.passed = False
+
+        # If this thread fails to terminate, don't hang the whole program
+        self.setDaemon(True)
+
+    def _rendezvous(self):
+        """Pause until all threads arrive here"""
+        s = self.state
+        s.arrived_threads_lock.acquire()
+        s.arrived_threads += 1
+        if s.arrived_threads == s.nthreads:
+            s.arrived_threads_lock.release()
+            s.ev_arrived.set()
+        else:
+            s.arrived_threads_lock.release()
+            s.ev_arrived.wait()
+
+    def run(self):
+        try:
+            self.before_rendezvous()
+        finally:
+            self._rendezvous()
+
+        # all threads have passed the rendezvous, wait for
+        # resume_after_rendezvous()
+        self.state.ev_resume.wait()
+
+        self.after_rendezvous()
+        self.passed = True
+
 def read_from_which_host(
     rsc,
     mode,
@@ -109,26 +225,26 @@ def assertReadFrom(testcase, rsc, member, *args, **kwargs):
     :Parameters:
       - `testcase`: A unittest.TestCase
       - `rsc`: A ReplicaSetConnection
-      - `member`: replica_set_connection.Member expected to be used
+      - `member`: A host:port expected to be used
       - `mode`: A ReadPreference
-      - `tag_sets`: List of dicts of tags for data-center-aware reads
-      - `secondary_acceptable_latency_ms`: a float
+      - `tag_sets` (optional): List of dicts of tags for data-center-aware reads
+      - `secondary_acceptable_latency_ms` (optional): a float
     """
     for _ in range(10):
         testcase.assertEqual(member, read_from_which_host(rsc, *args, **kwargs))
 
 def assertReadFromAll(testcase, rsc, members, *args, **kwargs):
     """Check that a query with the given mode, tag_sets, and
-       secondary_acceptable_latency_ms can read from any of a set of
-       replica-set members.
+    secondary_acceptable_latency_ms reads from all members in a set, and
+    only members in that set.
 
     :Parameters:
       - `testcase`: A unittest.TestCase
       - `rsc`: A ReplicaSetConnection
-      - `members`: Sequence of replica_set_connection.Member expected to be used
+      - `members`: Sequence of host:port expected to be used
       - `mode`: A ReadPreference
-      - `tag_sets`: List of dicts of tags for data-center-aware reads
-      - `secondary_acceptable_latency_ms`: a float
+      - `tag_sets` (optional): List of dicts of tags for data-center-aware reads
+      - `secondary_acceptable_latency_ms` (optional): a float
     """
     members = set(members)
     used = set()
